"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HAPProxy = void 0;
const assert_1 = __importDefault(require("assert"));
const definitions_1 = require("./definitions");
const filters_1 = require("./filters");
const HAPClient_1 = require("./HAPClient");
const HAPServer_1 = require("./HAPServer");
const http_protocol_1 = require("./lib/http-protocol");
const IdentifierCache_1 = require("./storage/IdentifierCache");
const hap_proxy_1 = require("./types/hap-proxy");
const uuid_1 = require("./utils/uuid");
class HAPProxy {
    constructor(clientInfo, accessoryInfo) {
        this.serviceFilterDefinitions = {}; // indexed by the ServiceType (uuid)
        this.serviceFilters = {}; // indexed by "aid.iid" (iid of the service)
        this.serverToClientConnections = {};
        this.client = new HAPClient_1.HAPClient(clientInfo);
        this.server = new HAPServer_1.HAPServer(accessoryInfo);
        this.identifierCache = new IdentifierCache_1.IdentifierCache(this.client.clientInfo);
        this.addServiceFilter(definitions_1.ServiceType.PROTOCOL_INFORMATION, filters_1.AccessoryInformationServiceFilter);
        this.addServiceFilter(definitions_1.ServiceType.DATA_STREAM_TRANSPORT_MANAGEMENT, filters_1.DataStreamTransportManagementServiceFilter);
        this.server.on(HAPServer_1.HAPServerEvents.CONNECTION, this.handleServerConnection.bind(this));
        this.server.on(HAPServer_1.HAPServerEvents.UNPAIRED, this.handleServerUnpaired.bind(this));
        this.server.on(HAPServer_1.HAPServerEvents.ACCESSORIES, this.handleServerAccessories.bind(this));
        this.server.on(HAPServer_1.HAPServerEvents.GET_CHARACTERISTICS, this.handleServerGetCharacteristics.bind(this));
        this.server.on(HAPServer_1.HAPServerEvents.SET_CHARACTERISTICS, this.handleServerSetCharacteristics.bind(this));
        this.server.on(HAPServer_1.HAPServerEvents.PREPARE_WRITE, this.handleServerPrepareWrite.bind(this));
        this.server.on(HAPServer_1.HAPServerEvents.RESOURCE, this.handleServerResource.bind(this));
        this.client.on(HAPClient_1.HAPClientEvents.CONFIG_NUMBER_CHANGE, (num) => {
            this.server.accessoryInfo.updateConfigNumber(num)
                .then(() => this.server.advertiser.updateAdvertisement());
        });
    }
    listen(targetPort) {
        this.identifierCache.load()
            .then(() => this.buildFiltersFromCache())
            .then(() => this.server.listen(targetPort))
            .then(() => this.client.bonjourBrowser.deviceInfoPromise())
            .then(info => this.server.accessoryInfo.initWithClientInfo(info))
            .then(() => this.server.advertiser.updateAdvertisement());
        // TODO add catch? would get thrown if client is not on network
    }
    addServiceFilter(serviceType, filterDefinition) {
        this.serviceFilterDefinitions[serviceType] = filterDefinition;
    }
    buildFiltersFromCache() {
        let built = 0;
        this.identifierCache.entries().forEach(entry => {
            const serviceFilter = this.getOrCreateServiceFilter(entry.aid, entry.serviceIid, entry.serviceType);
            if (serviceFilter) {
                const filter = this.getOrCreateCharacteristicFilter(serviceFilter, entry.characteristicIid, entry.characteristicType);
                if (filter) {
                    built++;
                }
            }
        });
        if (built > 0) {
            console.log("Constructed " + built + " characteristic filters from IdentifierCache!"); // TODO debug
        }
    }
    getOrCreateServiceFilter(aid, iid, type) {
        let filterInstance = this.serviceFilters[aid + "." + iid];
        if (!filterInstance) { // check if we have a definition for this service. If we have create a new instance from it.
            const definition = this.serviceFilterDefinitions[type];
            if (definition) {
                filterInstance = new definition(this, aid, iid);
                this.serviceFilters[aid + "." + iid] = filterInstance;
            }
            else {
                return undefined; // no filter for this service type defined
            }
        }
        return filterInstance;
    }
    getOrCreateCharacteristicFilter(parent, iid, type) {
        let filterInstance = parent.characteristicsFilters[iid];
        if (!filterInstance) {
            const definition = parent.characteristicFilterDefinitions[type];
            if (definition) {
                filterInstance = new definition(this, parent, iid);
                parent.characteristicsFilters[iid] = filterInstance;
            }
            else {
                return undefined; // no filter for this characteristic type defined
            }
        }
        return filterInstance;
    }
    getCharacteristicFilter(aid, iid) {
        // iid passed to this method is the instance id of the characteristic
        const serviceIid = this.identifierCache.lookupServiceIid(aid, iid);
        // something heavily went wrong if this entry is not in the cache
        assert_1.default(serviceIid !== undefined, "Corrupted identifierCache. iid of service not found");
        const serviceFilter = this.serviceFilters[aid + "." + serviceIid];
        if (!serviceFilter) {
            return undefined;
        }
        return serviceFilter.characteristicsFilters[iid];
    }
    handleServerConnection(serverConnection) {
        const clientConnection = this.client.newConnection();
        this.serverToClientConnections[serverConnection.sessionID] = clientConnection;
        clientConnection.on(HAPClient_1.HAPClientConnectionEvents.EVENT_RAW, this.forwardEvent.bind(this, serverConnection));
        clientConnection.on(HAPClient_1.HAPClientConnectionEvents.DISCONNECTED, this.handleClientDisconnected.bind(this, serverConnection));
        clientConnection.ensureConnected().catch(reason => {
            console.log("Terminating server connection again since client could not be connected: " + reason); // TODO adjust message
            this.handleClientDisconnected(serverConnection);
        });
        serverConnection.on(HAPServer_1.HAPServerConnectionEvents.DISCONNECTED, this.handleServerDisconnected.bind(this, serverConnection, clientConnection));
    }
    handleServerUnpaired() {
        // noinspection JSIgnoredPromiseFromCall
        this.client.newConnection().removePairing();
    }
    forwardEvent(serverConnection, eventBuf) {
        const event = JSON.parse(eventBuf.toString());
        event.characteristics.forEach(characteristic => {
            const type = this.identifierCache.lookupType(characteristic.aid, characteristic.iid);
            const serviceName = definitions_1.ServiceTypes.TYPE_TO_NAME[type.serviceType];
            const characteristicName = definitions_1.CharacteristicTypes.TYPE_TO_NAME[type.characteristicType];
            console.log(new Date().toISOString() + " Event sent for " + serviceName + "." + characteristicName + " with value '" + characteristic.value + "'");
        });
        serverConnection.sendRawEvent(eventBuf);
    }
    handleClientDisconnected(serverConnection) {
        serverConnection.disconnect();
        delete this.serverToClientConnections[serverConnection.sessionID];
    }
    handleServerDisconnected(serverConnection, clientConnection) {
        clientConnection.disconnect();
        delete this.serverToClientConnections[serverConnection.sessionID];
    }
    matchConnections(connection) {
        return this.serverToClientConnections[connection.sessionID];
    }
    handleServerAccessories(connection, callback) {
        const clientConnection = this.matchConnections(connection);
        clientConnection.accessories()
            .then(httpResponse => {
            if (httpResponse.status !== 200) {
                callback(undefined, HAPProxy.responseToServerResponse(httpResponse));
                return;
            }
            let chain = Promise.resolve();
            this.identifierCache.rebuild(); // clear cache before we rebuild it
            // TODO check if any services got removed and then remove the appropriate service filter
            const attributeDatabase = JSON.parse(httpResponse.body.toString());
            attributeDatabase.accessories.forEach(accessory => {
                accessory.services.forEach(service => {
                    const serviceType = uuid_1.uuid.toLongForm(service.type);
                    const serviceFilter = this.getOrCreateServiceFilter(accessory.aid, service.iid, serviceType);
                    service.characteristics.forEach(characteristic => {
                        const characteristicType = uuid_1.uuid.toLongForm(characteristic.type);
                        this.identifierCache.persists(accessory.aid, service.iid, characteristic.iid, serviceType, characteristicType);
                        if (!serviceFilter) {
                            return;
                        }
                        const filter = this.getOrCreateCharacteristicFilter(serviceFilter, characteristic.iid, characteristicType);
                        if (characteristic.value) { // only need to filter if we actually return a value
                            if (filter) {
                                chain = chain
                                    .then(() => filter.filterRead(connection, clientConnection, characteristic.value))
                                    .then(value => characteristic.value = value)
                                    .catch(reason => console.log("Filter caused error: " + reason)); // TODO adjust message
                            }
                        }
                    });
                });
            });
            return chain
                .then(() => this.identifierCache.save())
                .then(() => {
                httpResponse.body = Buffer.from(JSON.stringify(attributeDatabase));
                callback(undefined, HAPProxy.responseToServerResponse(httpResponse));
            });
        })
            .catch(reason => callback(new Error(reason)));
    }
    handleServerGetCharacteristics(connection, ids, callback) {
        const clientConnection = this.matchConnections(connection);
        clientConnection.getCharacteristicsRaw(ids)
            .then(httpResponse => {
            if (httpResponse.status !== 200) {
                callback(undefined, HAPProxy.responseToServerResponse(httpResponse));
                return;
            }
            let chain = Promise.resolve();
            let rebuildResponse = false;
            const response = JSON.parse(httpResponse.body.toString());
            response.characteristics.forEach(characteristic => {
                const type = this.identifierCache.lookupType(characteristic.aid, characteristic.iid);
                const serviceName = definitions_1.ServiceTypes.TYPE_TO_NAME[type.serviceType];
                const characteristicName = definitions_1.CharacteristicTypes.TYPE_TO_NAME[type.characteristicType];
                console.log(new Date().toISOString() + " " + serviceName + "." + characteristicName + " was read to be '" + characteristic.value + "'");
                const filter = this.getCharacteristicFilter(characteristic.aid, characteristic.iid);
                if (filter) {
                    const previousValue = characteristic.value;
                    chain = chain
                        .then(() => filter.filterRead(connection, clientConnection, previousValue))
                        .then(value => {
                        characteristic.value = value;
                        if (previousValue !== value) {
                            rebuildResponse = true;
                        }
                    })
                        .catch(reason => console.log("Filter caused error: " + reason)); // TODO adjust message
                }
            });
            return chain.then(() => {
                if (rebuildResponse) {
                    httpResponse.body = Buffer.from(JSON.stringify(response));
                }
                callback(undefined, HAPProxy.responseToServerResponse(httpResponse));
            });
        })
            .catch(reason => callback(new Error(reason)));
    }
    handleServerSetCharacteristics(connection, writeRequest, callback) {
        const clientConnection = this.matchConnections(connection);
        let chain = Promise.resolve();
        let rebuildRequest = false;
        let containsWriteResponse = false;
        const request = JSON.parse(writeRequest.toString());
        request.characteristics.forEach(characteristic => {
            if (characteristic.value !== undefined) {
                const type = this.identifierCache.lookupType(characteristic.aid, characteristic.iid);
                const serviceName = definitions_1.ServiceTypes.TYPE_TO_NAME[type.serviceType];
                const characteristicName = definitions_1.CharacteristicTypes.TYPE_TO_NAME[type.characteristicType];
                console.log(new Date().toISOString() + " " + serviceName + "." + characteristicName + " was set to '" + characteristic.value + "'");
                const filter = this.getCharacteristicFilter(characteristic.aid, characteristic.iid);
                if (filter) {
                    const previousValue = characteristic.value;
                    chain = chain
                        .then(() => filter.filterWrite(connection, clientConnection, previousValue))
                        .then(value => {
                        characteristic.value = value;
                        if (previousValue !== value) {
                            rebuildRequest = true;
                        }
                    })
                        .catch(reason => console.log("Filter caused error: " + reason)); // TODO adjust message
                }
            }
            if (characteristic.r) {
                containsWriteResponse = true;
            }
        });
        chain
            .then(() => {
            if (rebuildRequest) {
                writeRequest = Buffer.from(JSON.stringify(request));
            }
        })
            .then(() => clientConnection.setCharacteristicsRaw(writeRequest))
            .then(httpResponse => {
            let chain = Promise.resolve();
            let rebuildResponse = false;
            if (containsWriteResponse) { // only decode response if we actually queried an control point characteristic
                if (httpResponse.status !== http_protocol_1.HTTPStatus.MULTI_STATUS) { // prop. some none spec compliant accessory
                    console.log("WARNING: The accessory returned unexpected http status (" +
                        http_protocol_1.HTTPStatus[httpResponse.status] + "/" + httpResponse.status + ") when we where expecting a write response!");
                    return;
                }
                const response = JSON.parse(httpResponse.body.toString());
                response.characteristics.forEach(characteristic => {
                    if (characteristic.status !== 0) {
                        console.log("Got an error for our put request: " + hap_proxy_1.HAPStatusCode[characteristic.status]); // TODO message
                        return;
                    }
                    if (characteristic.value === undefined) {
                        return;
                    }
                    const type = this.identifierCache.lookupType(characteristic.aid, characteristic.iid);
                    const serviceName = definitions_1.ServiceTypes.TYPE_TO_NAME[type.serviceType];
                    const characteristicName = definitions_1.CharacteristicTypes.TYPE_TO_NAME[type.characteristicType];
                    console.log(serviceName + "." + characteristicName + " write response was '" + characteristic.value + "'");
                    const filter = this.getCharacteristicFilter(characteristic.aid, characteristic.iid);
                    if (filter) {
                        const previousValue = characteristic.value;
                        chain = chain
                            .then(() => filter.filterWriteResponse(connection, clientConnection, previousValue))
                            .then(value => {
                            characteristic.value = value;
                            if (previousValue !== value) {
                                rebuildResponse = true;
                            }
                        })
                            .catch(reason => console.log("Filter caused error: " + reason)); // TODO adjust message
                    }
                });
                chain.then(() => {
                    if (rebuildResponse) {
                        httpResponse.body = Buffer.from(JSON.stringify(response));
                    }
                });
            }
            return chain.then(() => callback(undefined, HAPProxy.responseToServerResponse(httpResponse)));
        })
            .catch(reason => callback(new Error(reason)));
    }
    handleServerPrepareWrite(connection, prepareRequest, callback) {
        // TODO adding ability to filter those too?
        this.matchConnections(connection).prepareWriteRaw(prepareRequest)
            .then(response => {
            callback(undefined, HAPProxy.responseToServerResponse(response));
        })
            .catch(reason => callback(new Error(reason)));
    }
    handleServerResource(connection, resourceRequest, callback) {
        // TODO adding ability to filter those too?
        this.matchConnections(connection).resourceRaw(resourceRequest)
            .then(response => {
            callback(undefined, HAPProxy.responseToServerResponse(response));
        })
            .catch(reason => callback(new Error(reason)));
    }
    static responseToServerResponse(response) {
        const contentType = response.headers["Content-Type"];
        delete response.headers["Content-Type"];
        return {
            status: response.status,
            contentType: contentType,
            headers: {},
            data: response.body,
        };
    }
}
exports.HAPProxy = HAPProxy;
//# sourceMappingURL=HAPProxy.js.map