"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataStreamConnection = exports.DataStreamConnectionEvents = exports.ConnectionState = void 0;
const EventEmitter_1 = require("../lib/EventEmitter");
const index_1 = require("./index");
const encryption = __importStar(require("../crypto/encryption"));
const assert_1 = __importDefault(require("assert"));
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default('DataStream:Connection');
var ConnectionState;
(function (ConnectionState) {
    ConnectionState[ConnectionState["UNINITIALIZED"] = 0] = "UNINITIALIZED";
    ConnectionState[ConnectionState["READY"] = 1] = "READY";
    ConnectionState[ConnectionState["CLOSING"] = 2] = "CLOSING";
    ConnectionState[ConnectionState["CLOSED"] = 3] = "CLOSED";
})(ConnectionState = exports.ConnectionState || (exports.ConnectionState = {}));
var DataStreamConnectionEvents;
(function (DataStreamConnectionEvents) {
    DataStreamConnectionEvents["PAYLOAD"] = "raw-payload";
    DataStreamConnectionEvents["MESSAGE"] = "message";
    DataStreamConnectionEvents["CLOSED"] = "closed";
})(DataStreamConnectionEvents = exports.DataStreamConnectionEvents || (exports.DataStreamConnectionEvents = {}));
class DataStreamConnection extends EventEmitter_1.EventEmitter {
    constructor(socket) {
        super();
        this.remoteAddress = "";
        this.state = ConnectionState.UNINITIALIZED;
        this.connected = false;
        this.socket = socket;
        this.socket.setNoDelay(true); // disable Nagle algorithm
        this.socket.setKeepAlive(true);
        this.encryptionNonce = 0;
        this.encryptionNonceBuffer = Buffer.alloc(8);
        this.decryptionNonce = 0;
        this.decryptionNonceBuffer = Buffer.alloc(8);
        this.socket.on('data', this.handleData.bind(this));
        this.socket.on('error', this.handleError.bind(this));
        this.socket.on('close', this.handleClose.bind(this)); // we MUST register for this event, otherwise the error will bubble up to the top and crash the node process entirely.
    }
    setKeys(encryptionKey, decryptionKey) {
        assert_1.default(this.state === ConnectionState.UNINITIALIZED);
        this.encryptionKey = encryptionKey;
        this.decryptionKey = decryptionKey;
        this.state = ConnectionState.READY;
    }
    handleInitialization(firstFrame) {
        throw new Error("Unsupported operation!");
    }
    handleData(data) {
        if (this.state >= ConnectionState.CLOSING) {
            return;
        }
        let frameIndex = 0;
        const frames = this.decodeHDSFrames(data);
        if (frames.length === 0) { // not enough data
            return;
        }
        if (this.state === ConnectionState.UNINITIALIZED) { // really only relevant for the server side
            // at the beginning we are only interested in trying to decrypt the first frame in order to test decryption keys
            const firstFrame = frames[frameIndex++];
            this.handleInitialization(firstFrame);
            if (this.state === ConnectionState.UNINITIALIZED) {
                // did not find a prepared session, server already closed this connection; nothing to do here
                return;
            }
        }
        for (; frameIndex < frames.length; frameIndex++) { // decrypt all remaining frames
            if (!this.decryptHDSFrame(frames[frameIndex])) {
                debug("[%s] HDS frame decryption or authentication failed. Connection will be terminated!", this.remoteAddress);
                this.close();
                return;
            }
        }
        frames.forEach(frame => this.emit(DataStreamConnectionEvents.PAYLOAD, frame.plaintextPayload));
        const messages = this.decodePayloads(frames); // decode contents of payload
        messages.forEach(message => this.emit(DataStreamConnectionEvents.MESSAGE, message));
    }
    decodeHDSFrames(data) {
        if (this.frameBuffer !== undefined) {
            data = Buffer.concat([this.frameBuffer, data]);
            this.frameBuffer = undefined;
        }
        const totalBufferLength = data.length;
        const frames = [];
        for (let frameBegin = 0; frameBegin < totalBufferLength;) {
            if (frameBegin + 4 > totalBufferLength) {
                // we don't have enough data in the buffer for the next header
                this.frameBuffer = data.slice(frameBegin);
                break;
            }
            const payloadType = data.readUInt8(frameBegin); // type defining structure of payload; 8-bit; currently expected to be 1
            const payloadLength = data.readUIntBE(frameBegin + 1, 3); // read 24-bit big-endian uint length field
            if (payloadLength > DataStreamConnection.MAX_PAYLOAD_LENGTH) {
                debug("[%s] Connection send payload with size bigger than the maximum allow for data stream", this.remoteAddress);
                this.close();
                return [];
            }
            const remainingBufferLength = totalBufferLength - frameBegin - 4; // subtract 4 for payloadType (1-byte) and payloadLength (3-byte)
            // check if the data from this frame is already there (payload + 16-byte authTag)
            if (payloadLength + 16 > remainingBufferLength) {
                // Frame is fragmented, so we wait until we receive more
                this.frameBuffer = data.slice(frameBegin);
                break;
            }
            const payloadBegin = frameBegin + 4;
            const authTagBegin = payloadBegin + payloadLength;
            const header = data.slice(frameBegin, payloadBegin); // header is also authenticated using authTag
            const cipheredPayload = data.slice(payloadBegin, authTagBegin);
            const plaintextPayload = Buffer.alloc(payloadLength);
            const authTag = data.slice(authTagBegin, authTagBegin + 16);
            frameBegin = authTagBegin + 16; // move to next frame
            if (payloadType === 1) {
                const hdsFrame = {
                    header: header,
                    cipheredPayload: cipheredPayload,
                    plaintextPayload: plaintextPayload,
                    authTag: authTag,
                };
                frames.push(hdsFrame);
            }
            else {
                debug("[%s] Encountered unknown payload type %d for payload: %s", this.remoteAddress, plaintextPayload.toString('hex'));
            }
        }
        return frames;
    }
    decryptHDSFrame(frame, keyOverwrite) {
        encryption.writeUInt64LE(this.decryptionNonce, this.decryptionNonceBuffer, 0); // update nonce buffer
        const key = keyOverwrite || this.decryptionKey;
        try {
            frame.plaintextPayload = encryption.chacha20_poly1305_decryptAndVerify(key, this.decryptionNonceBuffer, frame.header, frame.cipheredPayload, frame.authTag);
            this.decryptionNonce++; // we had a successful encryption, increment the nonce
            return true;
        }
        catch (error) {
            // frame decryption or authentication failed. Could happen when our guess for a PreparedDataStreamSession is wrong
            return false;
        }
    }
    decodePayloads(frames) {
        const messages = [];
        frames.forEach(frame => {
            const payload = frame.plaintextPayload;
            const headerLength = payload.readUInt8(0);
            const messageLength = payload.length - headerLength - 1;
            const headerBegin = 1;
            const messageBegin = headerBegin + headerLength;
            const headerPayload = new index_1.DataStreamReader(payload.slice(headerBegin, headerBegin + headerLength));
            const messagePayload = new index_1.DataStreamReader(payload.slice(messageBegin, messageBegin + messageLength));
            let headerDictionary;
            let messageDictionary;
            try {
                headerDictionary = index_1.DataStreamParser.decode(headerPayload);
                headerPayload.finished();
            }
            catch (error) {
                debug("[%s] Failed to decode header payload: %s", this.remoteAddress, error.message);
                return;
            }
            try {
                messageDictionary = index_1.DataStreamParser.decode(messagePayload);
                messagePayload.finished();
            }
            catch (error) {
                debug("[%s] Failed to decode message payload: %s (header: %o)", this.remoteAddress, error.message, headerDictionary);
                return;
            }
            let type;
            const protocol = headerDictionary["protocol"];
            let topic;
            let id = undefined;
            let status = undefined;
            if (headerDictionary["event"] !== undefined) {
                type = index_1.MessageType.EVENT;
                topic = headerDictionary["event"];
            }
            else if (headerDictionary["request"] !== undefined) {
                type = index_1.MessageType.REQUEST;
                topic = headerDictionary["request"];
                id = headerDictionary["id"];
            }
            else if (headerDictionary["response"] !== undefined) {
                type = index_1.MessageType.RESPONSE;
                topic = headerDictionary["response"];
                id = headerDictionary["id"];
                status = headerDictionary["status"];
            }
            else {
                debug("[%s] Encountered unknown payload header format: %o (message: %o)", this.remoteAddress, headerDictionary, messageDictionary);
                return;
            }
            const message = {
                type: type,
                protocol: protocol,
                topic: topic,
                id: id,
                status: status,
                message: messageDictionary,
            };
            messages.push(message);
        });
        return messages;
    }
    sendHDSFrame(header, message) {
        if (this.state >= ConnectionState.CLOSING) {
            throw Error("Cannot send message on closing/closed socket!");
        }
        const headerWriter = new index_1.DataStreamWriter();
        const messageWriter = new index_1.DataStreamWriter();
        index_1.DataStreamParser.encode(header, headerWriter);
        index_1.DataStreamParser.encode(message, messageWriter);
        const payloadHeaderBuffer = Buffer.alloc(1);
        payloadHeaderBuffer.writeUInt8(headerWriter.length(), 0);
        const payloadBuffer = Buffer.concat([payloadHeaderBuffer, headerWriter.getData(), messageWriter.getData()]);
        this.sendRawHDSFrame(payloadBuffer);
    }
    sendRawHDSFrame(payloadBuffer) {
        // TODO buffer frames if !connected
        if (payloadBuffer.length > DataStreamConnection.MAX_PAYLOAD_LENGTH) {
            throw new Error("Tried sending payload with length larger than the maximum allowed for data stream");
        }
        const frameTypeBuffer = Buffer.alloc(1);
        frameTypeBuffer.writeUInt8(1, 0);
        let frameLengthBuffer = Buffer.alloc(4);
        frameLengthBuffer.writeUInt32BE(payloadBuffer.length, 0);
        frameLengthBuffer = frameLengthBuffer.slice(1, 4); // a bit hacky but the only real way to write 24-bit int in node
        const frameHeader = Buffer.concat([frameTypeBuffer, frameLengthBuffer]);
        encryption.writeUInt64LE(this.encryptionNonce++, this.encryptionNonceBuffer);
        const encrypted = encryption.chacha20_poly1305_encryptAndSeal(this.encryptionKey, this.encryptionNonceBuffer, frameHeader, payloadBuffer);
        this.socket.write(Buffer.concat([frameHeader, encrypted.ciphertext, encrypted.authTag]));
        /* Useful for debugging outgoing packages and detecting encoding errors
        console.log("SENT DATA: " + payloadBuffer.toString("hex"));
        const frame: HDSFrame = {
            header: frameHeader,
            plaintextPayload: payloadBuffer,
            cipheredPayload: cipheredPayload,
            authTag: authTag,
        };
        const sentMessage = this.decodePayloads([frame])[0];
        console.log("Sent message: " + JSON.stringify(sentMessage, null, 4));
        //*/
    }
    close() {
        if (this.state >= ConnectionState.CLOSING) {
            return; // connection is already closing/closed
        }
        this.connected = false;
        this.state = ConnectionState.CLOSING;
        this.socket.end();
    }
    onHAPSessionClosed() {
        // If the hap session is closed it is probably also a good idea to close the data stream session
        debug("[%s] HAP session disconnected. Also closing DataStream connection now.", this.remoteAddress);
        this.close();
    }
    handleError(error) {
        debug("[%s] Encountered socket error: %s", this.remoteAddress, error.message);
        // handleClose() will be called next
    }
    handleClose() {
        this.state = ConnectionState.CLOSED;
        this.connected = false;
        this.emit(DataStreamConnectionEvents.CLOSED);
    }
}
exports.DataStreamConnection = DataStreamConnection;
DataStreamConnection.MAX_PAYLOAD_LENGTH = 0x11111111111111111111;
//# sourceMappingURL=DataStreamConnection.js.map