"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    Object.defineProperty(o, k2, { enumerable: true, get: function() { return m[k]; } });
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DataStreamServerConnection = exports.DataStreamServerConnectionEvents = exports.DataStreamServer = exports.DataStreamServerEvents = void 0;
const crypto_1 = __importDefault(require("crypto"));
const debug_1 = __importDefault(require("debug"));
const net_1 = __importDefault(require("net"));
const encryption = __importStar(require("../crypto/encryption"));
const HAPServer_1 = require("../HAPServer");
const EventEmitter_1 = require("../lib/EventEmitter");
const DataStreamConnection_1 = require("./DataStreamConnection");
const debug = debug_1.default('DataStream:Server');
var ServerState;
(function (ServerState) {
    ServerState[ServerState["UNINITIALIZED"] = 0] = "UNINITIALIZED";
    ServerState[ServerState["BINDING"] = 1] = "BINDING";
    ServerState[ServerState["LISTENING"] = 2] = "LISTENING";
    ServerState[ServerState["CLOSING"] = 3] = "CLOSING";
})(ServerState || (ServerState = {}));
var DataStreamServerEvents;
(function (DataStreamServerEvents) {
    DataStreamServerEvents["CONNECTION_OPENED"] = "connection-opened";
    DataStreamServerEvents["CONNECTION_IDENTIFIED"] = "connection-identified";
    DataStreamServerEvents["CONNECTION_CLOSED"] = "connection-closed";
})(DataStreamServerEvents = exports.DataStreamServerEvents || (exports.DataStreamServerEvents = {}));
/**
 * DataStreamServer which listens for incoming tcp connections and handles identification of new connections
 *
 * @event 'connection-opened': (connection: DataStreamServerConnection) => void
 *        This event is emitted when a new client socket is received. At this point we have no idea to what
 *        hap session this connection will be matched.
 *
 * @event 'connection-closed': (connection: DataStreamServerConnection) => void
 *        This event is emitted when the socket of a connection gets closed.
 */
class DataStreamServer extends EventEmitter_1.EventEmitter {
    constructor() {
        super();
        this.state = ServerState.UNINITIALIZED;
        this.preparedSessions = [];
        this.connections = [];
    }
    prepareSession(connection, controllerKeySalt, clientConnection, callback) {
        debug("Preparing for incoming HDS connection from session %s", connection.sessionID);
        const accessoryKeySalt = crypto_1.default.randomBytes(32);
        const salt = Buffer.concat([controllerKeySalt, accessoryKeySalt]);
        const accessoryToControllerInfo = Buffer.from("HDS-Read-Encryption-Key");
        const controllerToAccessoryInfo = Buffer.from("HDS-Write-Encryption-Key");
        const accessoryToControllerEncryptionKey = encryption.HKDF("sha512", salt, connection.encryptionContext.sharedSecret, accessoryToControllerInfo, 32);
        const controllerToAccessoryEncryptionKey = encryption.HKDF("sha512", salt, connection.encryptionContext.sharedSecret, controllerToAccessoryInfo, 32);
        const preparedSession = {
            connection: connection,
            hdsClientConnection: clientConnection,
            accessoryToControllerEncryptionKey: accessoryToControllerEncryptionKey,
            controllerToAccessoryEncryptionKey: controllerToAccessoryEncryptionKey,
            accessoryKeySalt: accessoryKeySalt,
            connectTimeout: setTimeout(() => this.timeoutPreparedSession(preparedSession), 10000),
        };
        this.preparedSessions.push(preparedSession);
        this.checkTCPServerEstablished(preparedSession, () => callback(preparedSession));
    }
    timeoutPreparedSession(preparedSession) {
        debug("Prepared HDS session timed out out since no connection was opened for 10 seconds (%s)", preparedSession.connection.sessionID);
        const index = this.preparedSessions.indexOf(preparedSession);
        if (index >= 0) {
            this.preparedSessions.splice(index, 1);
        }
        this.checkCloseable();
    }
    checkTCPServerEstablished(preparedSession, callback) {
        switch (this.state) {
            case ServerState.UNINITIALIZED:
                debug("Starting up TCP server.");
                this.tcpServer = net_1.default.createServer();
                this.tcpServer.once('listening', this.listening.bind(this, preparedSession, callback));
                this.tcpServer.on('connection', this.onConnection.bind(this));
                this.tcpServer.on('close', this.closed.bind(this));
                this.tcpServer.listen();
                this.state = ServerState.BINDING;
                break;
            case ServerState.BINDING:
                debug("TCP server already running. Waiting for it to bind.");
                this.tcpServer.once('listening', this.listening.bind(this, preparedSession, callback));
                break;
            case ServerState.LISTENING:
                debug("Instructing client to connect to already running TCP server");
                preparedSession.port = this.tcpPort;
                callback();
                break;
            case ServerState.CLOSING:
                debug("TCP socket is currently closing. Trying again when server is fully closed and opening a new one then.");
                this.tcpServer.once('close', () => setTimeout(() => this.checkTCPServerEstablished(preparedSession, callback), 10));
                break;
        }
    }
    listening(preparedSession, callback) {
        this.state = ServerState.LISTENING;
        const address = this.tcpServer.address();
        if (address && typeof address !== "string") { // address is only typeof string when listening to a pipe or unix socket
            this.tcpPort = address.port;
            preparedSession.port = address.port;
            debug("TCP server is now listening for new data stream connections on port %s", address.port);
            callback();
        }
    }
    onConnection(socket) {
        debug("[%s] New DataStream connection was established", socket.remoteAddress);
        const connection = new DataStreamServerConnection(socket);
        connection.on(DataStreamServerConnectionEvents.IDENTIFICATION, this.handleSessionIdentification.bind(this, connection));
        connection.on(DataStreamConnection_1.DataStreamConnectionEvents.CLOSED, this.connectionClosed.bind(this, connection));
        this.connections.push(connection);
        this.emit(DataStreamServerEvents.CONNECTION_OPENED, connection);
    }
    handleSessionIdentification(connection, firstFrame, callback) {
        let identifiedSession = undefined;
        for (let i = 0; i < this.preparedSessions.length; i++) {
            const preparedSession = this.preparedSessions[i];
            // if we successfully decrypt the first frame with this key we know to which session this connection belongs
            if (connection.decryptHDSFrame(firstFrame, preparedSession.controllerToAccessoryEncryptionKey)) {
                identifiedSession = preparedSession;
                break;
            }
        }
        callback(identifiedSession);
        if (identifiedSession) {
            debug("[%s] Connection was successfully identified (linked with sessionId: %s)", connection.remoteAddress, identifiedSession.connection.sessionID);
            const index = this.preparedSessions.indexOf(identifiedSession);
            if (index >= 0) {
                this.preparedSessions.splice(index, 1);
            }
            clearTimeout(identifiedSession.connectTimeout);
            identifiedSession.connectTimeout = undefined;
            this.emit(DataStreamServerEvents.CONNECTION_IDENTIFIED, identifiedSession, connection);
        }
        else {
            debug("[%s] Could not identify connection. Terminating.", connection.remoteAddress);
            connection.close(); // disconnecting since first message was not a valid hello
        }
    }
    connectionClosed(connection) {
        debug("[%s] DataStream connection closed", connection.remoteAddress);
        const index = this.connections.indexOf(connection);
        if (index >= 0) {
            this.connections.splice(index, 1);
        }
        this.emit(DataStreamServerEvents.CONNECTION_CLOSED, connection);
        this.checkCloseable();
    }
    checkCloseable() {
        if (this.connections.length === 0 && this.preparedSessions.length === 0) {
            debug("Last connection disconnected. Closing the server now.");
            this.state = ServerState.CLOSING;
            // noinspection JSIgnoredPromiseFromCall
            this.tcpServer.close();
        }
    }
    closed() {
        this.tcpServer = undefined;
        this.tcpPort = undefined;
        this.state = ServerState.UNINITIALIZED;
    }
}
exports.DataStreamServer = DataStreamServer;
var DataStreamServerConnectionEvents;
(function (DataStreamServerConnectionEvents) {
    DataStreamServerConnectionEvents["IDENTIFICATION"] = "identification";
})(DataStreamServerConnectionEvents = exports.DataStreamServerConnectionEvents || (exports.DataStreamServerConnectionEvents = {}));
/**
 * DataStream connection which holds any necessary state information, encryption an decryption keys, manages
 * protocol handlers and also handles sending and receiving of data stream frames.
 *
 * @event 'identification': (frame: HDSFrame, callback: IdentificationCallback) => void
 *        This event is emitted when the first HDSFrame is received from a new connection.
 *        The connection expects the handler to identify the connection by trying to match the decryption keys.
 *        If identification was successful the PreparedDataStreamSession should be supplied to the callback,
 *        otherwise undefined should be supplied.
 *
 * @event 'handle-message-globally': (message: DataStreamMessage) => void
 *        This event is emitted when no handler could be found for the given protocol of a event or request message.
 *
 * @event 'closed': () => void
 *        This event is emitted when the socket of the connection was closed.
 */
class DataStreamServerConnection extends DataStreamConnection_1.DataStreamConnection {
    constructor(socket) {
        super(socket);
        this.remoteAddress = socket.remoteAddress;
        this.connected = true;
    }
    handleInitialization(firstFrame) {
        this.emit(DataStreamServerConnectionEvents.IDENTIFICATION, firstFrame, (identifiedSession) => {
            if (identifiedSession) {
                // horray, we found our session
                this.connection = identifiedSession.connection;
                this.connection.on(HAPServer_1.HAPServerConnectionEvents.DISCONNECTED, this.onHAPSessionClosed.bind(this)); // register close listener
                this.setKeys(identifiedSession.accessoryToControllerEncryptionKey, identifiedSession.controllerToAccessoryEncryptionKey);
            }
        });
    }
}
exports.DataStreamServerConnection = DataStreamServerConnection;
//# sourceMappingURL=DataStreamServer.js.map