"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.HTTPRequestParser = exports.HTTPResponseParser = exports.HTTPParser = exports.HTTPRoutes = exports.HTTPContentType = exports.HAPPairingHTTPStatus = exports.HTTPStatus = exports.HTTPHeader = exports.HTTPMethod = exports.ParsingState = void 0;
const debug_1 = __importDefault(require("debug"));
const debug = debug_1.default("HTTP:ResponseParser");
var ParsingState;
(function (ParsingState) {
    ParsingState[ParsingState["HEAD"] = 0] = "HEAD";
    ParsingState[ParsingState["HEADERS"] = 1] = "HEADERS";
    ParsingState[ParsingState["RESPONSE_BODY"] = 2] = "RESPONSE_BODY";
})(ParsingState = exports.ParsingState || (exports.ParsingState = {}));
var HTTPMethod;
(function (HTTPMethod) {
    // noinspection JSUnusedGlobalSymbols
    HTTPMethod["GET"] = "GET";
    HTTPMethod["HEAD"] = "HEAD";
    HTTPMethod["POST"] = "POST";
    HTTPMethod["PUT"] = "PUT";
    HTTPMethod["DELETE"] = "DELETE";
    HTTPMethod["CONNECT"] = "CONNECT";
    HTTPMethod["OPTIONS"] = "options";
    HTTPMethod["TRACE"] = "TRACE";
})(HTTPMethod = exports.HTTPMethod || (exports.HTTPMethod = {}));
var HTTPHeader;
(function (HTTPHeader) {
    // noinspection JSUnusedGlobalSymbols
    HTTPHeader["CONTENT_TYPE"] = "Content-Type";
    HTTPHeader["DATE"] = "Date";
    HTTPHeader["CONNECTION"] = "Connection";
    HTTPHeader["TRANSFER_ENCODING"] = "Transfer-Encoding";
    HTTPHeader["CONTENT_LENGTH"] = "Content-Length";
})(HTTPHeader = exports.HTTPHeader || (exports.HTTPHeader = {}));
// 4xx or 5xx response must include an HAP status Code property
var HTTPStatus;
(function (HTTPStatus) {
    // noinspection JSUnusedGlobalSymbols
    HTTPStatus[HTTPStatus["SUCCESS"] = 200] = "SUCCESS";
    HTTPStatus[HTTPStatus["NO_CONTENT"] = 204] = "NO_CONTENT";
    HTTPStatus[HTTPStatus["MULTI_STATUS"] = 207] = "MULTI_STATUS";
    HTTPStatus[HTTPStatus["BAD_REQUEST"] = 400] = "BAD_REQUEST";
    HTTPStatus[HTTPStatus["NOT_FOUND"] = 404] = "NOT_FOUND";
    HTTPStatus[HTTPStatus["UNPROCESSABLE_ENTITY"] = 422] = "UNPROCESSABLE_ENTITY";
    HTTPStatus[HTTPStatus["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
    HTTPStatus[HTTPStatus["SERVICE_UNAVAILABLE"] = 503] = "SERVICE_UNAVAILABLE";
})(HTTPStatus = exports.HTTPStatus || (exports.HTTPStatus = {}));
var HAPPairingHTTPStatus;
(function (HAPPairingHTTPStatus) {
    HAPPairingHTTPStatus[HAPPairingHTTPStatus["OK"] = 200] = "OK";
    HAPPairingHTTPStatus[HAPPairingHTTPStatus["BAD_REQUEST"] = 400] = "BAD_REQUEST";
    HAPPairingHTTPStatus[HAPPairingHTTPStatus["METHOD_NOT_ALLOWED"] = 405] = "METHOD_NOT_ALLOWED";
    HAPPairingHTTPStatus[HAPPairingHTTPStatus["TOO_MANY_REQUEST"] = 429] = "TOO_MANY_REQUEST";
    HAPPairingHTTPStatus[HAPPairingHTTPStatus["CONNECTION_AUTHORIZATION_REQUIRED"] = 470] = "CONNECTION_AUTHORIZATION_REQUIRED";
    HAPPairingHTTPStatus[HAPPairingHTTPStatus["INTERNAL_SERVER_ERROR"] = 500] = "INTERNAL_SERVER_ERROR";
})(HAPPairingHTTPStatus = exports.HAPPairingHTTPStatus || (exports.HAPPairingHTTPStatus = {}));
var HTTPContentType;
(function (HTTPContentType) {
    HTTPContentType["TEXT_HTML"] = "text/html";
    HTTPContentType["HAP_JSON"] = "application/hap+json";
    HTTPContentType["PAIRING_TLV8"] = "application/pairing+tlv8";
})(HTTPContentType = exports.HTTPContentType || (exports.HTTPContentType = {}));
var HTTPRoutes;
(function (HTTPRoutes) {
    // noinspection JSUnusedGlobalSymbols
    HTTPRoutes["IDENTIFY"] = "/identify";
    HTTPRoutes["PAIR_SETUP"] = "/pair-setup";
    HTTPRoutes["PAIR_VERIFY"] = "/pair-verify";
    HTTPRoutes["PAIRINGS"] = "/pairings";
    HTTPRoutes["ACCESSORIES"] = "/accessories";
    HTTPRoutes["CHARACTERISTICS"] = "/characteristics";
    HTTPRoutes["PREPARE"] = "/prepare";
    HTTPRoutes["RESOURCE"] = "/resource";
})(HTTPRoutes = exports.HTTPRoutes || (exports.HTTPRoutes = {}));
class HTTPParser {
    constructor() {
        this.state = ParsingState.HEAD;
        this.readerIndex = 0;
        this.buffer = Buffer.alloc(0);
    }
    appendData(buffer) {
        this.buffer = Buffer.concat([this.buffer, buffer]);
    }
    readLine() {
        let previousWasR = false; // previous character was '\r'
        for (let i = this.readerIndex; i < this.buffer.length; i++) {
            const character = String.fromCharCode(this.buffer[i]);
            if (character === '\r') {
                previousWasR = true;
                continue;
            }
            else if (character === '\n') {
                const endIndex = previousWasR ? i - 1 : i;
                const line = this.buffer.slice(this.readerIndex, endIndex);
                this.readerIndex = i + 1;
                return line;
            }
            if (previousWasR) {
                previousWasR = false;
            }
        }
        return null;
    }
    readFixedLengthLine(length) {
        const minimumRequiredLength = this.readerIndex + length;
        if (minimumRequiredLength > this.buffer.length) {
            return null; // there is not even enough data to fit the chunk
        }
        const terminatorIndex = minimumRequiredLength;
        if (terminatorIndex <= this.buffer.length
            && String.fromCharCode(this.buffer[terminatorIndex]) === '\n') {
            // TODO handle illegal format
            const data = this.buffer.slice(this.readerIndex, minimumRequiredLength);
            this.readerIndex += length + 1;
            return data;
        }
        else if (terminatorIndex + 1 <= this.buffer.length
            && String.fromCharCode(this.buffer[terminatorIndex]) === "\r" && String.fromCharCode(this.buffer[terminatorIndex + 1]) === "\n") {
            // TODO handle illegal format
            const data = this.buffer.slice(this.readerIndex, minimumRequiredLength);
            this.readerIndex += length + 2;
            return data;
        }
        return null; // not enough data
    }
    readStringLine() {
        const line = this.readLine();
        return line !== null ? line.toString() : null;
    }
    readChunk() {
        const oldReaderIndex = this.readerIndex;
        const size = this.readStringLine();
        if (size === null) {
            this.readerIndex = oldReaderIndex; // reset index to the state before we read the size
            return null; // indicates not enough data
        }
        const sizeInt = parseInt(size, 16); // lol
        // sizeInt == 0 indicates end of chunked data
        const chunk = this.readFixedLengthLine(sizeInt);
        if (chunk === null) { // reset index to the state before we read the size
            this.readerIndex = oldReaderIndex;
        }
        return chunk;
    }
    readFixedLengthBuffer(length) {
        if (this.readerIndex + length > this.buffer.length) {
            return null; // not enough data
        }
        const data = this.buffer.slice(this.readerIndex, this.readerIndex + length);
        this.readerIndex += length;
        return data;
    }
}
exports.HTTPParser = HTTPParser;
class HTTPResponseParser extends HTTPParser {
    constructor() {
        super();
        this.currentResponse = {};
    }
    parse() {
        const finishedResponses = [];
        for (;;) {
            // noinspection FallThroughInSwitchStatementJS
            switch (this.state) {
                case ParsingState.HEAD:
                    const head = this.readStringLine();
                    if (head !== null) {
                        const match = HTTPResponseParser.HTTP_HEADER_PATTERN.exec(head);
                        if (!match) {
                            throw new Error("Unexpected http header format!");
                        }
                        let [, messageType, version, status, statusText] = match;
                        const httpStatus = parseInt(status);
                        this.currentResponse.messageType = messageType;
                        this.currentResponse.version = version;
                        this.currentResponse.status = httpStatus;
                        this.currentResponse.statusText = statusText;
                        this.state = ParsingState.HEADERS;
                        this.currentResponse.headers = {};
                        debug("Successfully parsed HEAD '%s'", head);
                        // fallthrough into next state
                    }
                    else {
                        return finishedResponses; // we couldn't finish parsing
                    }
                case ParsingState.HEADERS:
                    let headerLine;
                    while ((headerLine = this.readStringLine())) {
                        let [name, value] = headerLine.split(/: /, 2);
                        this.currentResponse.headers[name] = value;
                    }
                    if (headerLine === null) {
                        return finishedResponses; // we couldn't finish parsing
                    }
                    else {
                        debug("Successfully parsed headers: %o", this.currentResponse.headers);
                        this.state = ParsingState.RESPONSE_BODY;
                        // fallthrough into next state
                    }
                case ParsingState.RESPONSE_BODY:
                    if (this.currentResponse.status === HTTPStatus.NO_CONTENT) {
                        this.currentResponse.body = Buffer.alloc(0);
                    }
                    else {
                        if (this.currentResponse.headers[HTTPHeader.CONTENT_LENGTH]) { // typically returned by certified accessories
                            const length = parseInt(this.currentResponse.headers[HTTPHeader.CONTENT_LENGTH]);
                            const data = this.readFixedLengthBuffer(length);
                            if (data) {
                                this.currentResponse.body = data;
                            }
                            else { // we could not finish this response, abort
                                return finishedResponses;
                            }
                        }
                        else if (this.currentResponse.headers[HTTPHeader.TRANSFER_ENCODING] === "chunked") { // typically returned by hap-nodejs
                            if (!this.currentResponse.body) {
                                this.currentResponse.body = Buffer.alloc(0);
                            }
                            for (;;) {
                                const chunk = this.readChunk();
                                if (chunk === null) {
                                    return finishedResponses; // we couldn't finish parsing
                                }
                                if (chunk.length === 0) { // we finished parsing
                                    break;
                                }
                                this.currentResponse.body = Buffer.concat([
                                    this.currentResponse.body, chunk
                                ]);
                            }
                        }
                    }
                    finishedResponses.push(this.currentResponse);
                    // reset stuff
                    this.currentResponse = {};
                    this.buffer = this.buffer.slice(this.readerIndex, this.buffer.length); // drop read bytes
                    this.readerIndex = 0; // reset reader index
                    this.state = ParsingState.HEAD; // reset state
            }
        }
    }
}
exports.HTTPResponseParser = HTTPResponseParser;
HTTPResponseParser.HTTP_HEADER_PATTERN = /(HTTP|EVENT)\/(\d+\.\d+)\s+(\d{3})\s+(.*?)$/;
class HTTPRequestParser extends HTTPParser {
    constructor() {
        super();
        this.currentRequest = {};
    }
    parse() {
        const finishedResponses = [];
        for (;;) {
            // noinspection FallThroughInSwitchStatementJS
            switch (this.state) {
                case ParsingState.HEAD:
                    const head = this.readStringLine();
                    if (head !== null) {
                        const match = HTTPRequestParser.HTTP_HEADER_PATTERN.exec(head);
                        if (!match) {
                            throw new Error("Unexpected http header format!");
                        }
                        let [, method, uri, version] = match;
                        this.currentRequest.method = method;
                        this.currentRequest.uri = uri;
                        this.currentRequest.version = version;
                        this.state = ParsingState.HEADERS;
                        this.currentRequest.headers = {};
                        debug("Successfully parsed HEAD '%s'", head);
                        // fallthrough into next state
                    }
                    else {
                        return finishedResponses; // we couldn't finish parsing
                    }
                case ParsingState.HEADERS:
                    let headerLine;
                    while ((headerLine = this.readStringLine())) {
                        let [name, value] = headerLine.split(/: /, 2);
                        debug("Successfully parse header '%s'='%s'", name, value);
                        this.currentRequest.headers[name] = value;
                    }
                    if (headerLine === null) {
                        return finishedResponses; // we couldn't finish parsing
                    }
                    else {
                        debug("found headers %o", this.currentRequest.headers);
                        this.state = ParsingState.RESPONSE_BODY;
                        // fallthrough into next state
                    }
                case ParsingState.RESPONSE_BODY:
                    if (this.currentRequest.method === HTTPMethod.POST || this.currentRequest.method === HTTPMethod.PUT) {
                        if (this.currentRequest.headers[HTTPHeader.CONTENT_LENGTH]) { // typically returned by certified accessories
                            const length = parseInt(this.currentRequest.headers[HTTPHeader.CONTENT_LENGTH]);
                            const data = this.readFixedLengthBuffer(length);
                            if (data) {
                                this.currentRequest.body = data;
                            }
                            else { // we could not finish this response, abort
                                return finishedResponses;
                            }
                        }
                        else if (this.currentRequest.headers[HTTPHeader.TRANSFER_ENCODING] === "chunked") { // typically returned by hap-nodejs
                            this.currentRequest.body = Buffer.alloc(0);
                            for (;;) {
                                const chunk = this.readChunk();
                                if (chunk === null) {
                                    return finishedResponses; // we couldn't finish parsing
                                }
                                if (chunk.length === 0) { // we finished parsing
                                    break;
                                }
                                this.currentRequest.body = Buffer.concat([
                                    this.currentRequest.body, chunk
                                ]);
                            }
                        }
                    }
                    finishedResponses.push(this.currentRequest);
                    // reset stuff
                    this.currentRequest = {};
                    this.buffer = this.buffer.slice(this.readerIndex, this.buffer.length); // drop read bytes
                    this.readerIndex = 0; // reset reader index
                    this.state = ParsingState.HEAD; // reset state
            }
        }
    }
}
exports.HTTPRequestParser = HTTPRequestParser;
HTTPRequestParser.HTTP_HEADER_PATTERN = /(GET|HEAD|POST|PUT|DELETE|CONNECT|OPTIONS|TRACE)\s+(\/.*?)\s+HTTP\/(\d+\.\d+)$/;
//# sourceMappingURL=http-protocol.js.map