"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AccessoryInfo = exports.AccessoryInfoEvents = exports.PermissionTypes = void 0;
const storage_1 = require("./storage");
const tweetnacl_1 = __importDefault(require("tweetnacl"));
const crypto_1 = __importDefault(require("crypto"));
const setup_code_1 = require("../lib/setup-code");
const hap_proxy_1 = require("../types/hap-proxy");
const EventEmitter_1 = require("../lib/EventEmitter");
var PermissionTypes;
(function (PermissionTypes) {
    // noinspection JSUnusedGlobalSymbols
    PermissionTypes[PermissionTypes["USER"] = 0] = "USER";
    PermissionTypes[PermissionTypes["ADMIN"] = 1] = "ADMIN";
})(PermissionTypes = exports.PermissionTypes || (exports.PermissionTypes = {}));
var AccessoryInfoEvents;
(function (AccessoryInfoEvents) {
    AccessoryInfoEvents["REMOVED_CLIENT"] = "removed-client";
})(AccessoryInfoEvents = exports.AccessoryInfoEvents || (exports.AccessoryInfoEvents = {}));
class AccessoryInfo extends EventEmitter_1.EventEmitter {
    constructor(accessoryName, pincode) {
        super();
        this.category = hap_proxy_1.HAPAccessoryCategory.OTHER;
        this.accessoryId = "";
        this.displayName = "";
        this.pincode = "";
        this.longTermPublicKey = Buffer.alloc(0);
        this.longTermSecretKey = Buffer.alloc(0);
        this.pairedClients = {};
        this.pairedAdminClientCount = 0;
        this.configNumber = 1; // hap configuration number
        this.setupID = "";
        this.loaded = false;
        /**
         * Add a paired client to memory.
         * @param {string} clientId
         * @param {Buffer} publicKey
         * @param {PermissionTypes} permission
         */
        this.addPairedClient = (clientId, publicKey, permission) => {
            this.pairedClients[clientId] = {
                clientId: clientId,
                publicKey: publicKey,
                permission: permission
            };
            if (permission === PermissionTypes.ADMIN)
                this.pairedAdminClientCount++;
        };
        this.updatePermission = (username, permission) => {
            const pairingInformation = this.pairedClients[username];
            if (pairingInformation) {
                const oldPermission = pairingInformation.permission;
                pairingInformation.permission = permission;
                if (oldPermission === PermissionTypes.ADMIN && permission !== PermissionTypes.ADMIN) {
                    this.pairedAdminClientCount--;
                }
                else if (oldPermission !== PermissionTypes.ADMIN && permission === PermissionTypes.ADMIN) {
                    this.pairedAdminClientCount++;
                }
            }
        };
        this.listPairings = () => {
            const array = [];
            for (const clientId in this.pairedClients) {
                // noinspection JSUnfilteredForInLoop
                const pairingInformation = this.pairedClients[clientId];
                array.push(pairingInformation);
            }
            return array;
        };
        /**
         * Remove a paired client from memory.
         * @param controller - the session of the controller initiated the removal of the pairing
         * @param {string} clientId
         */
        this.removePairedClient = (controller, clientId) => {
            this._removePairedClient0(controller, clientId);
            if (this.pairedAdminClientCount === 0) { // if we don't have any admin clients left paired it is required to kill all normal clients
                for (const clientId0 in this.pairedClients) {
                    // noinspection JSUnfilteredForInLoop
                    this._removePairedClient0(controller, clientId0);
                }
            }
        };
        this._removePairedClient0 = (controller, clientId) => {
            if (this.pairedClients[clientId] && this.pairedClients[clientId].permission === PermissionTypes.ADMIN)
                this.pairedAdminClientCount--;
            delete this.pairedClients[clientId];
            this.emit(AccessoryInfoEvents.REMOVED_CLIENT, controller, clientId);
        };
        // noinspection JSUnusedGlobalSymbols
        /**
         * Check if clientId is paired
         * @param username
         */
        this.isPaired = (username) => {
            return !!this.pairedClients[username];
        };
        this.hasAdminPermissions = (clientId) => {
            if (!clientId)
                return false;
            const pairingInformation = this.pairedClients[clientId];
            return !!pairingInformation && pairingInformation.permission === PermissionTypes.ADMIN;
        };
        // Gets the public key for a paired client as a Buffer, or falsey value if not paired.
        this.getClientPublicKey = (clientId) => {
            const pairingInformation = this.pairedClients[clientId];
            if (pairingInformation) {
                return pairingInformation.publicKey;
            }
            else {
                return undefined;
            }
        };
        this.accessoryName = accessoryName;
        this.pincode = pincode || "";
    }
    initWithClientInfo(deviceInfo) {
        this.configNumber = deviceInfo.configNumber;
        this.category = deviceInfo.category;
        return this.save();
    }
    updateConfigNumber(configNumber) {
        this.configNumber = configNumber;
        return this.save();
    }
    hasPairings() {
        //return Object.keys(this.pairedClients).length > 0;
        // we have at least one admin controller if we have any paired clients; should be easier to calculate
        return this.pairedAdminClientCount > 0;
    }
    save() {
        return __awaiter(this, void 0, void 0, function* () {
            if (!this.loaded) {
                throw new Error("Tried saving AccessoryInfo before it was even loaded!");
            }
            const saved = {
                accessoryId: this.accessoryId,
                displayName: this.displayName,
                pincode: this.pincode,
                longTermPublicKey: this.longTermPublicKey.toString("hex"),
                longTermSecretKey: this.longTermSecretKey.toString("hex"),
                pairedClients: [],
                configNumber: this.configNumber,
                setupID: this.setupID,
            };
            for (const clientId in this.pairedClients) {
                // noinspection JSUnfilteredForInLoop
                const pairing = this.pairedClients[clientId];
                // noinspection JSUnfilteredForInLoop
                const savedPairing = {
                    clientId: clientId,
                    publicKey: pairing.publicKey.toString("hex"),
                    permission: pairing.permission,
                };
                // @ts-ignore
                saved.pairedClients.push(savedPairing);
            }
            const storageKey = storage_1.StorageManager.accessoryFormatPersistKey(this.accessoryName);
            yield storage_1.StorageManager.init();
            yield storage_1.StorageManager.setItem(storageKey, saved);
        });
    }
    load() {
        return __awaiter(this, void 0, void 0, function* () {
            const storageKey = storage_1.StorageManager.accessoryFormatPersistKey(this.accessoryName);
            yield storage_1.StorageManager.init();
            const saved = yield storage_1.StorageManager.getItem(storageKey);
            this.loaded = true;
            if (saved) {
                this.accessoryId = saved.accessoryId;
                this.displayName = saved.displayName;
                this.pincode = saved.pincode;
                this.longTermPublicKey = Buffer.from(saved.longTermPublicKey, "hex");
                this.longTermSecretKey = Buffer.from(saved.longTermSecretKey, "hex");
                this.configNumber = saved.configNumber;
                this.setupID = saved.setupID;
                const pairings = saved.pairedClients;
                pairings.forEach(savedPairing => {
                    const pairingInformation = {
                        clientId: savedPairing.clientId,
                        publicKey: Buffer.from(savedPairing.publicKey, "hex"),
                        permission: savedPairing.permission,
                    };
                    this.pairedClients[savedPairing.clientId] = pairingInformation;
                    if (pairingInformation.permission === PermissionTypes.ADMIN) {
                        this.pairedAdminClientCount++;
                    }
                });
            }
            else {
                this.accessoryId = AccessoryInfo.genRandomMac();
                // we need to create a network unique displayName
                this.displayName = this.accessoryName + " "
                    + crypto_1.default.createHash("sha512")
                        .update(this.accessoryId, 'utf8')
                        .digest("hex").slice(0, 4).toUpperCase();
                if (this.pincode === "") {
                    this.pincode = yield setup_code_1.SetupCodeGenerator.generate();
                    // TODO do we have a nice way for this?
                    console.log("Generated setup code: " + this.pincode);
                }
                const longTerm = tweetnacl_1.default.sign.keyPair(); // generate new lt key pair
                this.longTermPublicKey = Buffer.from(longTerm.publicKey);
                this.longTermSecretKey = Buffer.from(longTerm.secretKey);
                this.setupID = AccessoryInfo.generateSetupID();
                yield this.save();
            }
        });
    }
    static genRandomMac() {
        const buf = crypto_1.default.randomBytes(6);
        let output = "";
        for (let i = 0; i < 6; i++) {
            if (output.length > 0) {
                output += ":";
            }
            output += buf.toString("hex", i, i + 1).toUpperCase();
        }
        return output;
    }
}
exports.AccessoryInfo = AccessoryInfo;
AccessoryInfo.generateSetupID = () => {
    const chars = '0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ';
    const bytes = crypto_1.default.randomBytes(4);
    let setupID = '';
    for (let i = 0; i < 4; i++) {
        const index = bytes.readUInt8(i) % 26;
        setupID += chars.charAt(index);
    }
    return setupID;
};
//# sourceMappingURL=AccessoryInfo.js.map